@@ -19,6 +19,8 @@
 #include <sstream>
 #include <unordered_map>
 
+#include "common/memory.h"
+
 class debugging_c {
 protected:
   static bool ms_send_to_logger;
@@ -32,6 +34,8 @@ public:
   }
 
   static void hexdump(const void *buffer_to_dump, size_t lenth);
+  static void hexdump(memory_c const &buffer_to_dump, boost::optional<std::size_t> max_length = boost::none);
+  static void hexdump(memory_cptr const &buffer_to_dump, boost::optional<std::size_t> max_length = boost::none);
 
   static bool requested(const char *option, std::string *arg = nullptr);
   static bool requested(const std::string &option, std::string *arg = nullptr) {
