@@ -65,7 +65,8 @@ enum file_type_e {
   FILE_TYPE_VOBSUB,
   FILE_TYPE_WAV,
   FILE_TYPE_WAVPACK4,
-  FILE_TYPE_MAX = FILE_TYPE_WAVPACK4
+  FILE_TYPE_WEBVTT,
+  FILE_TYPE_MAX = FILE_TYPE_WEBVTT
 };
 
 struct file_type_t {
