@@ -66,10 +66,11 @@ enum mpeg_ts_stream_type_e {
   STREAM_AUDIO_AC3          = 0x81, // Audio AC-3 (A/52)
   STREAM_AUDIO_DTS          = 0x82, // Audio DTS
   STREAM_AUDIO_AC3_LOSSLESS = 0x83, // Audio AC-3 - Dolby lossless
-  STREAM_AUDIO_AC3_PLUS     = 0x84, // Audio AC-3 - Dolby Digital Plus (E-AC-3)
+  STREAM_AUDIO_EAC3         = 0x84, // Audio AC-3 - Dolby Digital Plus (E-AC-3)
   STREAM_AUDIO_DTS_HD       = 0x85, // Audio DTS HD
   STREAM_AUDIO_DTS_HD_MA    = 0x86, // Audio DTS HD Master Audio
-  STREAM_AUDIO_AC3_PLUS2    = 0xa1, // Audio AC-3 - Dolby Digital Plus (E-AC-3); secondary stream
+  STREAM_AUDIO_EAC3_ATSC    = 0x87, // Audio AC-3 - Dolby Digital Plus (E-AC-3) as defined in ATSC A/52:2012 Annex G
+  STREAM_AUDIO_EAC3_2       = 0xa1, // Audio AC-3 - Dolby Digital Plus (E-AC-3); secondary stream
   STREAM_AUDIO_DTS_HD2      = 0xa2, // Audio DTS HD Express; secondary stream
   STREAM_VIDEO_VC1          = 0xEA, // Video VC-1
   STREAM_SUBTITLES_HDMV_PGS = 0x90, // HDMV PGS subtitles
