@@ -45,7 +45,7 @@
 #include "output/p_pcm.h"
 #include "output/p_textsubs.h"
 #include "output/p_theora.h"
-#include "output/p_video.h"
+#include "output/p_video_for_windows.h"
 #include "output/p_vorbis.h"
 #include "output/p_vpx.h"
 
@@ -225,8 +225,8 @@ public:
   vp8_ogg_header_t vp8_header;
   unsigned int pixel_width, pixel_height;
   int64_t frame_rate_num, frame_rate_den;
-  unsigned int num_header_packets_skipped;
   int64_t frames_since_granulepos_change;
+  debugging_option_c debug;
 
 public:
   ogm_v_vp8_demuxer_c(ogm_reader_c *p_reader, ogg_packet &op);
@@ -239,6 +239,7 @@ public:
   virtual generic_packetizer_c *create_packetizer();
   virtual void process_page(int64_t granulepos);
   virtual std::pair<unsigned int, unsigned int> get_pixel_dimensions() const;
+  virtual bool is_header_packet(ogg_packet &op) override;
 };
 
 class ogm_s_kate_demuxer_c: public ogm_demuxer_c {
@@ -895,7 +896,6 @@ ogm_demuxer_c::ogm_demuxer_c(ogm_reader_c *p_reader)
   , serialno(0)
   , eos(0)
   , units_processed(0)
-  , num_header_packets(2)
   , num_non_header_packets(0)
   , headers_read(false)
   , first_granulepos(0)
@@ -955,33 +955,27 @@ ogm_demuxer_c::process_page(int64_t /* granulepos */) {
 
 void
 ogm_demuxer_c::process_header_page() {
+  if (headers_read)
+    return;
+
   ogg_packet op;
 
   while (ogg_stream_packetout(&os, &op) == 1) {
     eos |= op.e_o_s;
 
     if (!is_header_packet(op)) {
-      if (nh_packet_data.size() != num_non_header_packets) {
+      if (nh_packet_data.size() < num_non_header_packets) {
         nh_packet_data.push_back(memory_c::clone(op.packet, op.bytes));
         continue;
       }
 
-      mxwarn_tid(reader->m_ti.m_fname, track_id,
-                 Y("Missing header/comment packets for stream. This file is broken but should be muxed correctly. "
-                   "If not please contact the author Moritz Bunkus <moritz@bunkus.org>.\n"));
       headers_read = true;
       ogg_stream_reset(&os);
       return;
     }
 
     packet_data.push_back(memory_c::clone(op.packet, op.bytes));
-
-    eos |= op.e_o_s;
   }
-
-  if (   (packet_data.size()    == num_header_packets)
-      && (nh_packet_data.size() >= num_non_header_packets))
-    headers_read = true;
 }
 
 std::pair<unsigned int, unsigned int>
@@ -1108,8 +1102,7 @@ ogm_a_pcm_demuxer_c::create_packetizer() {
 ogm_a_vorbis_demuxer_c::ogm_a_vorbis_demuxer_c(ogm_reader_c *p_reader)
   : ogm_demuxer_c(p_reader)
 {
-  codec              = codec_c::look_up(codec_c::type_e::A_VORBIS);
-  num_header_packets = 3;
+  codec = codec_c::look_up(codec_c::type_e::A_VORBIS);
 }
 
 void
@@ -1168,8 +1161,7 @@ ogm_a_opus_demuxer_c::ogm_a_opus_demuxer_c(ogm_reader_c *p_reader)
   : ogm_demuxer_c(p_reader)
   , m_calculated_end_timecode{timestamp_c::ns(0)}
 {
-  codec              = codec_c::look_up(codec_c::type_e::A_OPUS);
-  num_header_packets = 2;
+  codec = codec_c::look_up(codec_c::type_e::A_OPUS);
 }
 
 generic_packetizer_c *
@@ -1341,7 +1333,7 @@ ogm_v_mscomp_demuxer_c::create_packetizer() {
   if (mpeg4::p2::is_fourcc(sth->subtype))
     ptzr_obj = new mpeg4_p2_video_packetizer_c(reader, m_ti, fps, width, height, false);
   else
-    ptzr_obj = new video_packetizer_c(reader, m_ti, nullptr, fps, width, height);
+    ptzr_obj = new video_for_windows_packetizer_c(reader, m_ti, fps, width, height);
 
   show_packetizer_info(m_ti.m_id, ptzr_obj);
 
@@ -1408,8 +1400,7 @@ ogm_v_mscomp_demuxer_c::process_page(int64_t granulepos) {
 ogm_v_theora_demuxer_c::ogm_v_theora_demuxer_c(ogm_reader_c *p_reader)
   : ogm_demuxer_c(p_reader)
 {
-  codec              = codec_c::look_up(codec_c::type_e::V_THEORA);
-  num_header_packets = 3;
+  codec = codec_c::look_up(codec_c::type_e::V_THEORA);
 
   memset(&theora, 0, sizeof(theora_identification_header_t));
 }
@@ -1487,11 +1478,10 @@ ogm_v_vp8_demuxer_c::ogm_v_vp8_demuxer_c(ogm_reader_c *p_reader,
   , pixel_height(0)
   , frame_rate_num(0)
   , frame_rate_den(0)
-  , num_header_packets_skipped(1)
   , frames_since_granulepos_change(0)
+  , debug{"ogg_vp8|ogm_vp8"}
 {
-  codec              = codec_c::look_up(codec_c::type_e::V_VP8);
-  num_header_packets = 2;
+  codec = codec_c::look_up(codec_c::type_e::V_VP8);
 
   memcpy(&vp8_header, op.packet, sizeof(vp8_ogg_header_t));
 }
@@ -1539,41 +1529,44 @@ ogm_v_vp8_demuxer_c::create_packetizer() {
 
 void
 ogm_v_vp8_demuxer_c::process_page(int64_t granulepos) {
+  if ((0 < granulepos) && (granulepos != last_granulepos))
+    last_granulepos = granulepos;
+
   ogg_packet op;
+  std::vector<memory_cptr> packets;
 
   while (ogg_stream_packetout(&os, &op) == 1) {
     eos |= op.e_o_s;
 
-    if (num_header_packets_skipped < num_header_packets) {
-      ++num_header_packets_skipped;
-      continue;
-    }
+    if ((units_processed > 0) || !is_header_packet(op))
+      packets.push_back(memory_c::clone(op.packet, op.bytes));
+  }
 
-    if ((0 < granulepos) && (granulepos != last_granulepos)) {
-      last_granulepos                = granulepos;
-      frames_since_granulepos_change = 0;
-    }
+  if (packets.empty())
+    return;
 
-    // Zero-length frames are 'repeat previous frame' markers and
-    // cannot be I frames.
-    bool is_keyframe  = (0 != op.bytes) && ((op.packet[0] & 0x01) == 0);
-    int64_t pts       = granulepos >> 32;
-    int64_t inv_count = (granulepos >> 30) & 0x03;
-    int64_t distance  = (granulepos >>  3) & 0x07ffffff;
-    int64_t frame_num = pts - inv_count - 1 + frames_since_granulepos_change;
-    int64_t timecode  = (int64_t)(1000000000.0 * frame_num * frame_rate_den / frame_rate_num);
-    int64_t bref      = is_keyframe ? VFT_IFRAME : VFT_PFRAMEAUTOMATIC;
+  auto pts         = last_granulepos >> 32;
+  auto inv_count   = (last_granulepos >> 30) & 0x03;
+  auto distance    = (last_granulepos >>  3) & 0x07ffffff;
+  auto num_packets = packets.size();
+
+  for (auto idx = 0u; idx < num_packets; ++idx) {
+    // Zero-length packets are 'repeat previous frame' markers and
+    // cannot be I packets.
+    auto &data       = packets[idx];
+    auto is_keyframe = (0 != data->get_size()) && ((data->get_buffer()[0] & 0x01) == 0);
+    auto frame_num   = pts - std::min<int64_t>(pts, num_packets - idx - 1 + 1);
+    auto timecode    = static_cast<int64_t>(1000000000.0 * frame_num * frame_rate_den / frame_rate_num);
+    auto bref        = is_keyframe ? VFT_IFRAME : VFT_PFRAMEAUTOMATIC;
 
     ++units_processed;
-    ++frames_since_granulepos_change;
 
-    reader->m_reader_packetizers[ptzr]->process(new packet_t(new memory_c(op.packet, op.bytes, false), timecode, default_duration, bref, VFT_NOBFRAME));
+    reader->m_reader_packetizers[ptzr]->process(new packet_t(data, timecode, default_duration, bref, VFT_NOBFRAME));
 
-    mxverb(3,
-           boost::format("VP8 track %1% size %10% #proc %11% frame# %12% fr_num %2% fr_den %3% granulepos 0x%|4$08x| %|5$08x| pts %6% inv_count %7% distance %8%%9%\n")
-           % track_id % frame_rate_num % frame_rate_den % (granulepos >> 32) % (granulepos & 0xffffffff)
-           % pts % inv_count % distance
-           % (is_keyframe ? " key" : "") % op.bytes % units_processed % frame_num);
+    mxdebug_if(debug,
+               boost::format("VP8 track %1% size %10% #proc %11% frame# %12% fr_num %2% fr_den %3% granulepos 0x%|4$08x| %|5$08x| pts %6% inv_count %7% distance %8%%9%\n")
+               % track_id % frame_rate_num % frame_rate_den % pts % (last_granulepos & 0xffffffff)
+               % pts % inv_count % distance % (is_keyframe ? " key" : "") % data->get_size() % units_processed % frame_num);
   }
 }
 
@@ -1583,13 +1576,27 @@ ogm_v_vp8_demuxer_c::get_pixel_dimensions()
   return { pixel_width, pixel_height };
 }
 
+bool
+ogm_v_vp8_demuxer_c::is_header_packet(ogg_packet &op) {
+  if (   (static_cast<std::size_t>(op.bytes) >= sizeof(vp8_ogg_header_t))
+      && (op.packet[0]                 == 0x4f)
+      && (get_uint32_be(&op.packet[1]) == 0x56503830))
+    return true;
+
+  if (   (op.bytes     >= 7)
+      && (op.packet[0] == 0x03)
+      && (!std::memcmp(op.packet, "vorbis", 6)))
+    return true;
+
+  return false;
+}
+
 // -----------------------------------------------------------
 
 ogm_s_kate_demuxer_c::ogm_s_kate_demuxer_c(ogm_reader_c *p_reader)
   : ogm_demuxer_c(p_reader)
 {
-  codec              = codec_c::look_up(codec_c::type_e::S_KATE);
-  num_header_packets = 1; /* at least 1, will be updated upon reading the ID header */
+  codec = codec_c::look_up(codec_c::type_e::S_KATE);
 
   memset(&kate, 0, sizeof(kate_identification_header_t));
 }
@@ -1599,7 +1606,6 @@ ogm_s_kate_demuxer_c::initialize() {
   try {
     memory_cptr &mem = packet_data[0];
     kate_parse_identification_header(mem->get_buffer(), mem->get_size(), kate);
-    num_header_packets = kate.nheaders;
   } catch (mtx::kate::header_parsing_x &e) {
     mxerror_tid(reader->m_ti.m_fname, track_id, boost::format(Y("The Kate identification header could not be parsed (%1%).\n")) % e.error());
   }
