@@ -1563,8 +1563,8 @@ generic_packetizer_c::create_track_number() {
 }
 
 file_status_e
-generic_packetizer_c::read() {
-  return m_reader->read(this);
+generic_packetizer_c::read(bool force) {
+  return m_reader->read(this, force);
 }
 
 void
