@@ -222,6 +222,22 @@ Tab::addAttachments(QStringList const &fileNames) {
       continue;
     }
 
+    auto existingFileName = findExistingAttachmentFileName(info.fileName());
+    if (existingFileName) {
+      auto const answer = Util::MessageBox::question(this)
+        ->title(QY("Attachment with same name present"))
+        .text(Q("%1 %2")
+              .arg(QY("An attachment with the name '%1' is already present.").arg(*existingFileName))
+              .arg(QY("Do you really want to add '%1' as another attachment?").arg(QDir::toNativeSeparators(fileName))))
+        .buttonLabel(QMessageBox::Yes, QY("&Add attachment"))
+        .buttonLabel(QMessageBox::No,  QY("&Skip file"))
+        .defaultButton(QMessageBox::No)
+        .exec();
+
+      if (answer == QMessageBox::No)
+        continue;
+    }
+
     attachmentsToAdd << std::make_shared<Attachment>(fileName);
     attachmentsToAdd.back()->guessMIMEType();
   }
@@ -325,23 +341,30 @@ Tab::enableAttachmentControls(bool enable) {
 
 void
 Tab::enableAttachedFilesActions() {
-  auto hasSelection = !ui->attachedFiles->selectionModel()->selection().isEmpty();
+  auto numSelected  = selectedAttachedFiles().size();
+  auto hasSelection = !!numSelected;
   auto hasEntries   = !!m_attachedFilesModel->rowCount();
 
   m_enableSelectedAttachedFilesAction->setEnabled(hasSelection);
   m_disableSelectedAttachedFilesAction->setEnabled(hasSelection);
   m_enableAllAttachedFilesAction->setEnabled(hasEntries);
   m_disableAllAttachedFilesAction->setEnabled(hasEntries);
+
+  m_enableSelectedAttachedFilesAction->setText(QNY("&Enable selected attached file", "&Enable selected attached files", numSelected));
+  m_disableSelectedAttachedFilesAction->setText(QNY("&Disable selected attached file", "&Disable selected attached files", numSelected));
 }
 
 void
 Tab::enableAttachmentsActions() {
-  auto hasSelection = !ui->attachments->selectionModel()->selection().isEmpty();
+  auto numSelected  = selectedAttachments().size();
+  auto hasSelection = !!numSelected;
   auto hasEntries   = !!m_attachmentsModel->rowCount();
 
   m_removeAttachmentsAction->setEnabled(hasSelection);
   m_removeAllAttachmentsAction->setEnabled(hasEntries);
   m_selectAllAttachmentsAction->setEnabled(hasEntries);
+
+  m_removeAttachmentsAction->setText(QNY("&Remove attachment", "&Remove attachments", numSelected));
 }
 
 void
@@ -379,13 +402,10 @@ Tab::retranslateAttachmentsUI() {
   resizeAttachedFilesColumnsToContents();
   resizeAttachmentsColumnsToContents();
 
-  m_enableSelectedAttachedFilesAction->setText(QY("&Enable selected attached files"));
-  m_disableSelectedAttachedFilesAction->setText(QY("&Disable selected attached files"));
   m_enableAllAttachedFilesAction->setText(QY("E&nable all attached files"));
   m_disableAllAttachedFilesAction->setText(QY("Di&sable all attached files"));
 
   m_addAttachmentsAction->setText(QY("&Add attachments"));
-  m_removeAttachmentsAction->setText(QY("&Remove attachments"));
   m_removeAllAttachmentsAction->setText(QY("Remove a&ll attachments"));
   m_selectAllAttachmentsAction->setText(QY("&Select all attachments"));
 
@@ -473,4 +493,33 @@ Tab::attachedFileItemChanged(QStandardItem *item) {
   m_attachedFilesModel->attachedFileUpdated(*attachedFile);
 }
 
+boost::optional<QString>
+Tab::findExistingAttachmentFileName(QString const &fileName) {
+  auto lowerFileName = fileName.toLower();
+
+  for (int row = 0, numRows = m_attachedFilesModel->rowCount(); row < numRows; ++row) {
+    auto attachedFile = m_attachedFilesModel->attachedFileForRow(row);
+
+    if (!attachedFile)
+      continue;
+
+    auto existingFileName = QFileInfo{attachedFile->m_name}.fileName();
+    if (lowerFileName == existingFileName.toLower())
+      return existingFileName;
+  }
+
+  for (int row = 0, numRows = m_attachmentsModel->rowCount(); row < numRows; ++row) {
+    auto attachment = m_attachmentsModel->attachmentForRow(row);
+
+    if (!attachment)
+      continue;
+
+    auto existingFileName = QFileInfo{attachment->m_fileName}.fileName();
+    if (lowerFileName == existingFileName.toLower())
+      return existingFileName;
+  }
+
+  return {};
+}
+
 }}}
