@@ -1082,9 +1082,11 @@ Tab::addOrAppendFiles(bool append,
 
   QList<SourceFilePtr> identifiedFiles;
   for (auto &fileName : toIdentify) {
-    Util::FileIdentifier identifier{ this, fileName };
+    Util::FileIdentifier identifier{fileName};
     if (identifier.identify())
       identifiedFiles << identifier.file();
+    else
+      Util::MessageBox::critical(this)->title(identifier.errorTitle()).text(identifier.errorText()).exec();
   }
 
   if (!append)
@@ -1223,7 +1225,7 @@ Tab::enableMoveFilesButtons() {
 
 void
 Tab::enableFilesActions() {
-  int numSelected      = ui->files->selectionModel()->selection().size();
+  int numSelected      = selectedSourceFiles().size();
   bool hasRegularTrack = false;
   if (1 == numSelected)
     hasRegularTrack = m_config.m_files.end() != brng::find_if(m_config.m_files, [](SourceFilePtr const &file) { return file->hasRegularTrack(); });
@@ -1237,11 +1239,14 @@ Tab::enableFilesActions() {
   m_removeFilesAction->setEnabled(0 < numSelected);
   m_removeAllFilesAction->setEnabled(!m_config.m_files.isEmpty());
   m_openFilesInMediaInfoAction->setEnabled(0 < numSelected);
+
+  m_removeFilesAction->setText(QNY("&Remove file", "&Remove files", numSelected));
+  m_openFilesInMediaInfoAction->setText(QNY("Open file in &MediaInfo", "Open files in &MediaInfo", numSelected));
 }
 
 void
 Tab::enableTracksActions() {
-  int numSelected = ui->tracks->selectionModel()->selection().size();
+  int numSelected = selectedTracks().size();
   bool hasTracks  = !!m_tracksModel->rowCount();
 
   m_selectAllTracksAction->setEnabled(hasTracks);
@@ -1254,6 +1259,8 @@ Tab::enableTracksActions() {
   m_selectAllSubtitlesTracksAction->setEnabled(hasTracks);
 
   m_openTracksInMediaInfoAction->setEnabled(0 < numSelected);
+
+  m_openTracksInMediaInfoAction->setText(QNY("Open track's file in &MediaInfo", "Open tracks' files in &MediaInfo", numSelected));
 }
 
 void
@@ -1271,15 +1278,12 @@ Tab::retranslateInputUI() {
   m_addAdditionalPartsAction ->setText(QY("Add files as a&dditional parts"));
   m_addAdditionalPartsAction2->setText(QY("Add files as a&dditional parts"));
 
-  m_removeFilesAction->setText(QY("&Remove files"));
   m_removeAllFilesAction->setText(QY("Remove a&ll files"));
-  m_openFilesInMediaInfoAction->setText(QY("Open in &MediaInfo"));
 
   m_selectAllTracksAction->setText(QY("&Select all tracks"));
   m_selectTracksOfTypeMenu->setTitle(QY("Select all tracks of specific &type"));
   m_enableAllTracksAction->setText(QY("&Enable all tracks"));
   m_disableAllTracksAction->setText(QY("&Disable all tracks"));
-  m_openTracksInMediaInfoAction->setText(QY("Open in &MediaInfo"));
 
   m_selectAllVideoTracksAction->setText(QY("&Video"));
   m_selectAllAudioTracksAction->setText(QY("&Audio"));
@@ -1387,7 +1391,7 @@ Tab::setOutputFileNameMaybe() {
   // Don't override custom changes to the output file name.
   if (   !currentOutput.isEmpty()
       && !m_config.m_destinationAuto.isEmpty()
-      && (currentOutput != m_config.m_destinationAuto))
+      && (QDir::toNativeSeparators(currentOutput) != QDir::toNativeSeparators(m_config.m_destinationAuto)))
     return;
 
   if (Util::Settings::ToPreviousDirectory == policy)
