@@ -10,6 +10,7 @@
 #include "mkvtoolnix-gui/app.h"
 #include "mkvtoolnix-gui/forms/merge/tool.h"
 #include "mkvtoolnix-gui/forms/main_window/main_window.h"
+#include "mkvtoolnix-gui/jobs/tool.h"
 #include "mkvtoolnix-gui/merge/tab.h"
 #include "mkvtoolnix-gui/merge/tool.h"
 #include "mkvtoolnix-gui/main_window/main_window.h"
@@ -81,7 +82,7 @@ void
 Tool::enableMenuActions() {
   auto mwUi   = MainWindow::getUi();
   auto tab    = currentTab();
-  auto hasTab = !!tab;
+  auto hasTab = tab && tab->isEnabled();
 
   mwUi->actionMergeSave->setEnabled(hasTab);
   mwUi->actionMergeSaveAs->setEnabled(hasTab);
@@ -90,8 +91,8 @@ Tool::enableMenuActions() {
   mwUi->actionMergeStartMuxing->setEnabled(hasTab);
   mwUi->actionMergeAddToJobQueue->setEnabled(hasTab);
   mwUi->actionMergeShowMkvmergeCommandLine->setEnabled(hasTab);
-  mwUi->actionMergeCopyFirstFileNameToTitle->setEnabled(tab && tab->hasSourceFiles());
-  mwUi->actionMergeCopyOutputFileNameToTitle->setEnabled(tab && tab->hasDestinationFileName());
+  mwUi->actionMergeCopyFirstFileNameToTitle->setEnabled(hasTab && tab->hasSourceFiles());
+  mwUi->actionMergeCopyOutputFileNameToTitle->setEnabled(hasTab && tab->hasDestinationFileName());
 }
 
 void
@@ -166,6 +167,11 @@ Tool::openConfigFile(QString const &fileName) {
     cfg.m_lastConfigDir = QFileInfo{fileName}.path();
   });
 
+  if (MainWindow::jobTool()->addJobFile(fileName)) {
+    MainWindow::get()->setStatusBarMessage(QY("The job has been added to the job queue."));
+    return;
+  }
+
   auto tab = currentTab();
   if (tab && tab->isEmpty())
     tab->deleteLater();
@@ -275,14 +281,14 @@ Tool::showCommandLine() {
 void
 Tool::copyFirstFileNameToTitle() {
   auto tab = currentTab();
-  if (tab)
+  if (tab && tab->isEnabled())
     tab->onCopyFirstFileNameToTitle();
 }
 
 void
 Tool::copyOutputFileNameToTitle() {
   auto tab = currentTab();
-  if (tab)
+  if (tab && tab->isEnabled())
     tab->onCopyOutputFileNameToTitle();
 }
 
